#!/usr/bin/env python
"""\
Usage: %prog RULES

RULES is a file defining a set of diazo rules in css syntax, e.g:

<rules xmlns="http://namespaces.plone.org/diazo"
       xmlns:css="http://namespaces.plone.org/diazo/css">
       <copy css:content="#content-wrapper" css:theme="#page-content"/>
</rules>\
"""
from __future__ import absolute_import
from optparse import OptionParser
from lxml import etree
from experimental.cssselect import css_to_xpath
from . import utils
import sys
import logging

logger = logging.getLogger('diazo')
usage = __doc__


def convert_css_selectors(rules):
    """Convert css rules to xpath rules element tree in place
    """
    # XXX: There is a
    # :root pseudo-class - http://www.w3.org/TR/css3-selectors/#root-pseudo
    # We may wish to add support to lxml.cssselect for it some day.
    for element in rules.xpath("//@*[namespace-uri()='%s']/.." %
                               utils.namespaces['css']):
        tag_namespace = utils.namespace(element.tag)
        css_prefix = element.attrib.get(utils.fullname(utils.namespaces['css'],
                                                       'prefix'), None)
        for name, value in element.attrib.items():
            if not name.startswith('{%s}' % utils.namespaces['css']):
                continue
            localname = utils.localname(name)
            if localname == 'prefix':
                continue
            if not value:
                element.attrib[localname] = ""
                continue
            if css_prefix is not None:
                prefix = css_prefix
            elif (tag_namespace == utils.namespaces['diazo'] and
                  localname in ('content', 'content-children', 'if-content',
                                'if-not-content') or
                    (tag_namespace == utils.namespaces['xsl'] and
                     localname in ('match',))):
                prefix = '//'
            else:
                prefix = 'descendant-or-self::'
            element.attrib[localname] = css_to_xpath(value, prefix=prefix)

    return rules


def main():
    """Called from console script
    """
    parser = OptionParser(usage=usage)
    parser.add_option("-o", "--output", metavar="output.html",
                      help="Output filename (instead of stdout)",
                      dest="output", default=sys.stdout)
    parser.add_option("-p", "--pretty-print", action="store_true",
                      help="Pretty print output",
                      dest="pretty_print", default=False)
    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.error("Invalid number of arguments")
    rules = etree.parse(args[0])
    convert_css_selectors(rules)
    rules.write(options.output, pretty_print=options.pretty_print)


if __name__ == '__main__':
    main()



def alltests():
    import unittest
    from . import (
        test_trace,
        test_wsgi,
        test_diazo,
    )
    modules = [test_trace, test_wsgi, test_diazo]
    return unittest.TestSuite([module.test_suite() for module in modules])



#!/usr/bin/env python
import logging

from diazo.rules import process_rules
from diazo.utils import pkg_xsl

from lxml import etree

logger = logging.getLogger('diazo')

_runtrace_to_html = pkg_xsl('runtrace_to_html.xsl')


def log_to_xml_string(error_log):
    return """
<runtrace xmlns:css="http://namespaces.plone.org/diazo/css">%s</runtrace>
    """ % "".join(l.message for l in error_log
                  if l.message.startswith('<runtrace '))


def generate_runtrace(rules, error_log, rules_parser=None):
    """Annotate a rules file with the results of a transformation"""
    def condition_name(trace):
        """Generate attribute name for this entry"""
        for k in trace.attrib.keys():
            if(k == 'theme_xmlid'):
                continue
            if(k.startswith('{http://namespaces.plone.org/diazo/css}')):
                continue
            return "runtrace-" + k

    rules_doc = process_rules(rules, rules_parser=rules_parser,
                              stop='add_identifiers')
    trace_doc = etree.XML(log_to_xml_string(error_log))

    for trace in trace_doc.xpath('/runtrace/runtrace'):
        for el in rules_doc.xpath("id('" + trace.attrib['theme_xmlid'] + "')"):
            el.set(condition_name(trace), trace.text or '')
    return rules_doc


def runtrace_to_html(runtrace_doc):
    """Convert the runtrace document into HTML"""
    return _runtrace_to_html(runtrace_doc)


def error_log_to_html(error_log):
    """Convert an error log into an HTML representation"""
    doc = etree.Element('ul')
    for l in error_log:
        if l.message.startswith('<runtrace '):
            continue
        el = etree.Element('li')
        el.attrib['class'] = "domain_%s level_%s type_%s" % (l.domain_name,
                                                             l.level_name,
                                                             l.type_name)
        el.text = "%s [%d:%d]" % (l.message, l.line, l.column)
        doc.append(el)
    return doc


def generate_debug_html(base_url, rules=None, error_log=None,
                        rules_parser=None):
    """Generate an HTML node with debug info"""
    def newElement(tag, content, **kwargs):
        n = etree.Element(tag, **kwargs)
        if hasattr(content, 'tag'):
            n.append(content)
        else:
            n.text = content
        return n

    debug_output_iframe = etree.Element('div', id="diazo-debug-iframe")
    debug_output_iframe.attrib['style'] = "display:none"
    debug_output_iframe.attrib['data-iframe'] = "diazo-debug"
    debug_output_iframe.attrib['data-iframe-style'] = ""
    debug_output_iframe.attrib['data-iframe-position'] = "bottom"
    debug_output_iframe.attrib['data-iframe-resources'] = \
        base_url + '/diazo-debug.css;' + \
        base_url + '/jquery-1.8.3.min.js;' + \
        base_url + '/diazo-debug.js'
    debug_output = etree.Element('div', id="diazo-debug")

    if error_log:
        debug_output.append(newElement('section',
                            error_log_to_html(error_log),
                            id="diazo_error_log"))

    try:
        runtrace_doc = generate_runtrace(rules, error_log, rules_parser)
        debug_output.append(newElement('section',
                            runtrace_to_html(runtrace_doc).getroot(),
                            id="diazo_runtrace"))
    except etree.XMLSyntaxError:
        debug_output.append(newElement('section',
                                       "Rules document could not be parsed!",
                                       id="diazo_runtrace"))

    debug_output_iframe.append(debug_output)
    debug_output_iframe.append(newElement('script', " ",
                               text="text/javascript",
                               src=base_url + '/iframe.js'))
    return debug_output_iframe



#!/usr/bin/env python
"""\
Usage: %prog [-r] RULES

Preprocess RULES, an diazo rules file
"""
import logging
import re
from lxml import etree
from six import string_types
from future.moves.urllib.parse import urljoin
from future.moves.urllib.request import urlopen
from diazo.cssrules import convert_css_selectors
from diazo.utils import namespaces, fullname, pkg_xsl, _createOptionParser

logger = logging.getLogger('diazo')
usage = __doc__

IMPORT_STYLESHEET = re.compile(
    r'''(?P<before>@import[ \t]+(?P<paren>url\([ \t]?)?(?P<quote>['"]?))'''
    r'''(?P<url>\S+)(?P<after>(?P=quote)(?(paren)\)))''', re.IGNORECASE)
CONDITIONAL_SRC = re.compile(
    r'''(?P<before><[^>]*?(src|href)=(?P<quote>['"]?))'''
    r'''(?P<url>[^ \t\n\r\f\v>]+)(?P<after>(?P=quote)[^>]*?>)''',
    re.IGNORECASE)


update_transform = pkg_xsl('update-namespace.xsl')
normalize_rules = pkg_xsl('normalize-rules.xsl')
apply_conditions = pkg_xsl('apply-conditions.xsl')
merge_conditions = pkg_xsl('merge-conditions.xsl')
annotate_themes = pkg_xsl('annotate-themes.xsl')
annotate_rules = pkg_xsl('annotate-rules.xsl')
apply_rules = pkg_xsl('apply-rules.xsl')
fixup_themes = pkg_xsl('fixup-themes.xsl')


def anchor_safe_urljoin(base, url):
    """Join the base with the url only when the url doesn't start with '#'"""
    if url.startswith('#'):
        return url
    else:
        return urljoin(base, url)


def add_identifiers(rules_doc):
    """Add identifiers to the rules for debugging"""
    for i, elem in enumerate(rules_doc.xpath(
            '//diazo:rules | //diazo:rules/diazo:*'
            ' | //old1:rules | //old1:rules/old1:*'
            ' | //old2:rules | //old2:rules/old1:*', namespaces=namespaces)):
        elem.set(fullname(namespaces['xml'], 'id'), 'r%s' % i)
    return rules_doc


def update_namespace(rules_doc):
    """Convert old namespace to new namespace in place
    """
    update = False
    for ns in (namespaces['old1'], namespaces['old2']):
        if rules_doc.xpath("//*[namespace-uri()='%s']" % ns):
            logger.warning('The %s namespace is deprecated, use %s instead.' %
                           (ns, namespaces['diazo']))
            update = True
    for ns in (namespaces['oldcss1'], namespaces['oldcss2']):
        if rules_doc.xpath("//@*[namespace-uri()='%s']" % ns):
            logger.warning('The %s namespace is deprecated, use %s instead.' %
                           (ns, namespaces['css']))
            update = True
    if update:
        new_doc = update_transform(rules_doc)
        # Place the nodes into the old tree to preserve any custom resolvers
        new = new_doc.getroot()
        root = rules_doc.getroot()
        root.clear()
        root.tag = new.tag
        root.nsmap.update(new.nsmap.items())
        root.attrib.update(new.attrib.items())
        root.text = new.text
        root[:] = new[:]
        root.tail = new.tail
    return rules_doc


def escape_curly_brackets(theme_doc):
    for node in theme_doc.iter():
        for attr in node.attrib:
            if '{' in node.attrib[attr]:
                node.attrib[attr] = node.attrib[attr].replace('{', '{{')
            if '}' in node.attrib[attr]:
                node.attrib[attr] = node.attrib[attr].replace('}', '}}')


def expand_theme(element, theme_doc, absolute_prefix):
    prefix = urljoin(absolute_prefix, element.get('prefix', ''))
    apply_absolute_prefix(theme_doc, prefix)
    escape_curly_brackets(theme_doc)
    theme_root = theme_doc.getroot()
    preceding = list(theme_root.itersiblings(preceding=True))
    preceding.reverse()
    following = list(theme_root.itersiblings(preceding=False))
    element.extend(preceding)
    element.append(theme_root)
    element.extend(following)


def expand_themes(rules_doc, parser=None, absolute_prefix=None,
                  read_network=False):
    """Expand <theme href='...'/> nodes with the theme html.
    """
    if absolute_prefix is None:
        absolute_prefix = ''
    base = rules_doc.docinfo.URL
    if parser is None:
        parser = etree.HTMLParser()
    for element in rules_doc.xpath('//diazo:theme[@href]',
                                   namespaces=namespaces):
        url = urljoin(base, element.get('href'))
        if not read_network and \
                url.startswith(('ftp://', 'ftps://', 'http://', 'https://')):
            raise ValueError("Supplied theme '%s', "
                             "but network access denied." % url)
        elif read_network and \
                url.startswith(('ftp://', 'ftps://', 'http://', 'https://')):
            theme = urlopen(url)
        else:
            theme = url
        theme_doc = etree.parse(theme, parser=parser, base_url=url)
        expand_theme(element, theme_doc, absolute_prefix)
    return rules_doc


def apply_absolute_prefix(theme_doc, absolute_prefix):
    if not absolute_prefix:
        return
    if not absolute_prefix.endswith('/'):
        absolute_prefix = absolute_prefix + '/'
    for node in theme_doc.xpath('//*[@src]'):
        url = urljoin(absolute_prefix, node.get('src'))
        node.set('src', url)
    for node in theme_doc.xpath('//*[@href]'):
        url = anchor_safe_urljoin(absolute_prefix, node.get('href'))
        node.set('href', url)
    for node in theme_doc.xpath('//style'):
        if node.text is None:
            continue
        node.text = IMPORT_STYLESHEET.sub(
            lambda match: match.group('before') + urljoin(
                absolute_prefix, match.group('url')) + match.group('after'),
            node.text)
    for node in theme_doc.xpath('//comment()[starts-with(., "[if")]'):
        node.text = IMPORT_STYLESHEET.sub(
            lambda match: match.group('before') + urljoin(
                absolute_prefix, match.group('url')) + match.group('after'),
            node.text)
        node.text = CONDITIONAL_SRC.sub(
            lambda match: match.group('before') + urljoin(
                absolute_prefix, match.group('url')) + match.group('after'),
            node.text)


def add_extra(rules_doc, extra):
    root = rules_doc.getroot()
    extra_elements = extra.xpath('/xsl:stylesheet/xsl:*',
                                 namespaces=namespaces)
    root.extend(extra_elements)
    return rules_doc


def add_theme(rules_doc, theme, parser=None, absolute_prefix=None,
              read_network=False):
    if not read_network and \
            isinstance(theme, string_types) and \
            theme[:6] in ('ftp://', 'http:/', 'https:'):
        raise ValueError("Supplied theme '%s', "
                         "but network access denied." % theme)
    if absolute_prefix is None:
        absolute_prefix = ''
    if parser is None:
        parser = etree.HTMLParser()
    root = rules_doc.getroot()
    element = root.makeelement(fullname(namespaces['diazo'], 'theme'))
    root.append(element)
    theme_doc = etree.parse(theme, parser=parser)
    expand_theme(element, theme_doc, absolute_prefix)
    return rules_doc


def fixup_theme_comment_selectors(rules):
    """Comments must be converted to <xsl:comment> to be output, doing it early
    allows them to get an xml:id so they can be matched in the theme. The theme
    selector needs rewriting to replace comment() with xsl:comment
    """
    for element in rules.xpath("//@theme[contains(., 'comment()')]/.."):
        element.attrib['theme'] = element.attrib['theme'].replace(
            'comment()', 'xsl:comment')
    return rules


def process_rules(rules, theme=None, extra=None, trace=None, css=True,
                  xinclude=True, absolute_prefix=None, includemode=None,
                  update=True, parser=None, rules_parser=None,
                  read_network=False, stop=None):
    if trace:
        trace = '1'
    else:
        trace = '0'
    if rules_parser is None:
        rules_parser = etree.XMLParser(recover=False)
    rules_doc = etree.parse(rules, parser=rules_parser)
    if stop == 0:
        return rules_doc
    if parser is None:
        parser = etree.HTMLParser()
    if xinclude:
        # XXX: read_network limitation not yet supported
        #   for xinclude
        rules_doc.xinclude()
    if stop == 1:
        return rules_doc
    rules_doc = add_identifiers(rules_doc)
    if stop == 2 or stop == 'add_identifiers':
        return rules_doc
    if update:
        rules_doc = update_namespace(rules_doc)
    if stop == 3:
        return rules_doc
    if css:
        rules_doc = convert_css_selectors(rules_doc)
    if stop == 4:
        return rules_doc
    rules_doc = fixup_theme_comment_selectors(rules_doc)
    if stop == 5:
        return rules_doc
    rules_doc = expand_themes(rules_doc, parser, absolute_prefix, read_network)
    if theme is not None:
        rules_doc = add_theme(rules_doc, theme, parser, absolute_prefix,
                              read_network)
    if stop == 6:
        return rules_doc
    if includemode is None:
        includemode = 'document'
    includemode = "'%s'" % includemode
    rules_doc = normalize_rules(rules_doc, includemode=includemode)
    if stop == 7:
        return rules_doc
    rules_doc = apply_conditions(rules_doc)
    if stop == 8:
        return rules_doc
    rules_doc = merge_conditions(rules_doc)
    if stop == 9:
        return rules_doc
    rules_doc = fixup_themes(rules_doc)
    if stop == 10:
        return rules_doc
    rules_doc = annotate_themes(rules_doc)
    if stop == 11:
        return rules_doc
    rules_doc = annotate_rules(rules_doc)
    if stop == 12:
        return rules_doc
    rules_doc = apply_rules(rules_doc, trace=trace)
    return rules_doc


def main():
    """Called from console script
    """
    parser = _createOptionParser(usage=usage)
    parser.add_option("-s", "--stop", metavar="n", type="int",
                      help="Stop preprocessing at stage n",
                      dest="stop", default=None)
    (options, args) = parser.parse_args()

    if options.rules is None:
        if len(args) == 2 and options.theme is None:
            options.rules, options.theme = args
        elif len(args) == 1:
            options.rules, = args
        else:
            parser.error("Wrong number of arguments.")
    elif args:
        parser.error("Wrong number of arguments.")

    if options.trace:
        logger.setLevel(logging.DEBUG)

    rules_doc = process_rules(
        options.rules,
        theme=options.theme,
        extra=options.extra,
        trace=options.trace,
        absolute_prefix=options.absolute_prefix,
        includemode=options.includemode,
        read_network=options.read_network,
        stop=options.stop,
    )
    root = rules_doc.getroot()
    if not root.tail:
        root.tail = '\n'
    rules_doc.write(options.output, pretty_print=options.pretty_print)


if __name__ == '__main__':
    main()



import sys
import os.path

try:
    import unittest2 as unittest
except ImportError:
    import unittest

if __name__ == '__main__':
    __file__ = sys.argv[0]


def testfile(filename):
    return '/'.join(('file://',) + os.path.split(os.path.abspath(
        os.path.dirname(__file__))) + ('test_wsgi_files', filename,))

HTML = b"""\
<html>
    <body>
        <h1>Content title</h1>
        <div id="content">Content content</div>
    </body>
</html>
"""

HTML_ALTERNATIVE = b"""\
<html>
    <body>
        <h1>Content title</h1>
        <div id="content">Alternative content</div>
    </body>
</html>
"""

XSLT = b"""\
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    version="1.0"
    exclude-result-prefixes="xhtml">
    <xsl:template match="/">
    <html>
        <head>
            <title>Transformed</title>
        </head>
        <body>
            <xsl:copy-of select="//div[@id='content']" />
        </body>
    </html>
    </xsl:template>
</xsl:stylesheet>
"""

XSLT_XHTML = b"""\
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    version="1.0"
    exclude-result-prefixes="xhtml">
  <xsl:output method="xml" indent="no" omit-xml-declaration="yes"
    media-type="application/xhtml+xml" encoding="UTF-8"
    doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"/>
  <xsl:template match="/">
    <html>
      <head>
        <title>Transformed</title>
      </head>
      <body>
        <xsl:copy-of select="//div[@id='content']" />
        <br/>
      </body>
    </html>
  </xsl:template>
</xsl:stylesheet>
"""

XSLT_HTML = b"""\
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    version="1.0"
    exclude-result-prefixes="xhtml">
  <xsl:output method="html" indent="no" omit-xml-declaration="yes"
      media-type="text/html" encoding="UTF-8"
      doctype-public="-//W3C//DTD HTML 4.01//EN"
      doctype-system="http://www.w3.org/TR/html4/strict.dtd"/>
  <xsl:template match="/">
    <html>
        <head>
            <title>Transformed</title>
        </head>
        <body>
            <xsl:copy-of select="//div[@id='content']" />
            <br/>
        </body>
    </html>
  </xsl:template>
</xsl:stylesheet>
"""

# Note that this can only work with an html output method. Setting the doctype
# on the middleware along with the xml output method and an XHTML 1.0 doctype
# in the stylesheet is required for XHTML compatible output.
XSLT_HTML5 = b"""\
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    version="1.0"
    exclude-result-prefixes="xhtml">
    <xsl:template match="/">
    <xsl:text
      disable-output-escaping="yes">&lt;!DOCTYPE html&gt;&#10;</xsl:text>
    <html>
        <head>
            <title>Transformed</title>
        </head>
        <body>
            <xsl:copy-of select="//div[@id='content']" />
            <br/>
        </body>
    </html>
    </xsl:template>
</xsl:stylesheet>
"""

XSLT_PARAM = b"""\
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    version="1.0"
    exclude-result-prefixes="xhtml">
    <xsl:param name="someparam">defaultvalue</xsl:param>
    <xsl:template match="/">
    <html>
        <head>
            <title>Transformed</title>
        </head>
        <body>
            <xsl:copy-of select="//div[@id='content']" />
            <p><xsl:value-of select="$someparam" /></p>
        </body>
    </html>
    </xsl:template>
</xsl:stylesheet>
"""


class TestXSLTMiddleware(unittest.TestCase):

    def test_transform_filename(self):
        import tempfile
        import os

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        _, filename = tempfile.mkstemp()
        with open(filename, 'wb') as fp:
            fp.write(XSLT)

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, filename=filename)
        os.unlink(filename)

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers['Content-Type'],
                         'text/html; charset=UTF-8')
        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_transform_tree(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers['Content-Type'],
                         'text/html; charset=UTF-8')
        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_head_request(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('Content-Length', str(len(HTML)))]
            start_response(status, response_headers)
            return ['']  # Empty response for HEAD request

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT))

        env = dict(REQUEST_METHOD='HEAD')
        request = Request.blank('/', environ=env)
        # The *real* test is whether or not an exception is raised here.
        response = request.get_response(app)

        # Response headers for HEAD request must be updated.
        self.assertEqual(response.headers['Content-Type'],
                         'text/html; charset=UTF-8')
        self.assertEqual(response.headers.get('Content-Length'), None)
        self.assertFalse(response.body)

    def test_update_content_length(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('Content-Length', str(len(HTML)))]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT),
                             update_content_length=True)

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers['Content-Length'], '178')

    def test_dont_update_content_length(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('Content-Length', '1')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers.get('Content-Length'), None)

    def test_content_length_zero(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('Content-Length', '0')]
            start_response(status, response_headers)
            return ['']

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT),
                             update_content_length=True)

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers['Content-Length'], '0')

    def test_content_empty(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('Content-MD5',
                                    'd41d8cd98f00b204e9800998ecf8427e')]
            start_response(status, response_headers)
            return [b'']

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT),
                             update_content_length=True)

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers['Content-MD5'],
                         'd41d8cd98f00b204e9800998ecf8427e')

    def test_content_range(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            content_length = len(HTML)
            content_range = 'bytes %d-%d/%d' % (0,
                                                content_length - 1,
                                                content_length)
            response_headers = [('Content-Type', 'text/html'),
                                ('Content-Range', content_range),
                                ('Content-Length', str(content_length))]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertFalse('Content-Range' in response.headers)

    def test_no_content_length(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT),
                             set_content_length=False)

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertFalse('Content-Length' in response.headers)

    def test_doctype_html(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers['Content-Type'],
                         'text/html; charset=UTF-8')

    def test_doctype_xhtml(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {},
                             tree=etree.fromstring(XSLT_XHTML))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.headers['Content-Type'],
                         'application/xhtml+xml; charset=UTF-8')

    def test_doctype_html5(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {},
                             tree=etree.fromstring(XSLT_XHTML),
                             doctype="<!DOCTYPE html>")

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(response.body.startswith(b"<!DOCTYPE html>\n<html"))

    def test_ignored_extension(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT),
                             ignored_extensions=('html',))

        request = Request.blank('/index.html')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_diazo_off_request_header(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        request.headers['X-Diazo-Off'] = 'yes'
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        request = Request.blank('/')
        request.headers['X-Diazo-Off'] = 'no'
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_diazo_off_response_header(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('X-Diazo-Off', 'yes')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('X-Diazo-Off', 'no')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_non_html_content_type(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/plain')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_content_encoding(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html'),
                                ('Content-Encoding', 'zip')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_301(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '301 MOVED PERMANENTLY'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_302(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '302 MOVED'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_304(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '304 NOT MODIFIED'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_204(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '204 NO CONTENT'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_401(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application1(environ, start_response):
            status = '401 UNAUTHORIZED'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application1, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertEqual(response.body, HTML)

        def application2(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application2, {}, tree=etree.fromstring(XSLT))

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_html_serialization(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {}, tree=etree.fromstring(XSLT_HTML))
        request = Request.blank('/')
        response = request.get_response(app)

        # HTML serialisation
        self.assertTrue(
            b'<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" '
            b'"http://www.w3.org/TR/html4/strict.dtd">' in response.body)
        self.assertTrue(b'<br>' in response.body)

        app = XSLTMiddleware(application, {},
                             tree=etree.fromstring(XSLT_XHTML))
        request = Request.blank('/')
        response = request.get_response(app)

        # XHTML serialisation
        self.assertTrue(
            b'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" '
            b'"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
            in response.body)
        self.assertTrue(b'<br />' in response.body)

        app = XSLTMiddleware(application, {},
                             tree=etree.fromstring(XSLT_HTML5))
        request = Request.blank('/')
        response = request.get_response(app)

        # HTML 5 serialisation
        self.assertTrue(b'<!DOCTYPE html>' in response.body)
        self.assertTrue(b'<br/>' in response.body)

    def test_environ_param(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(
            application, {}, tree=etree.fromstring(XSLT_PARAM),
            environ_param_map={'test.param1': 'someparam'})

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(b'<p>defaultvalue</p>' in response.body)

        request = Request.blank('/')
        request.environ['test.param1'] = 'value1'
        response = request.get_response(app)

        self.assertTrue(b'<p>value1</p>' in response.body)

    def test_params(self):
        from lxml import etree

        from diazo.wsgi import XSLTMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = XSLTMiddleware(application, {},
                             tree=etree.fromstring(XSLT_PARAM))
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(b'<p>defaultvalue</p>' in response.body)

        app = XSLTMiddleware(application, {},
                             tree=etree.fromstring(XSLT_PARAM),
                             someparam='value1')
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(b'<p>value1</p>' in response.body)


class TestDiazoMiddleware(unittest.TestCase):

    def test_simple_transform(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = DiazoMiddleware(application, {},
                              testfile('simple_transform.xml'))
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_doctype_html5(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = DiazoMiddleware(application, {},
                              testfile('simple_transform.xml'),
                              doctype="<!DOCTYPE html>")
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(response.body.startswith(b"<!DOCTYPE html>\n<html"))

    def test_with_theme(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = DiazoMiddleware(application, {}, testfile('explicit_theme.xml'),
                              theme=testfile('theme.html'))
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_absolute_prefix(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = DiazoMiddleware(application, {},
                              testfile('simple_transform.xml'))
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)
        self.assertTrue(
            b'<link rel="stylesheet" href="./theme.css" />' in response.body)

        app = DiazoMiddleware(application, {},
                              testfile('simple_transform.xml'),
                              prefix='/static')
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)
        self.assertTrue(
            b'<link rel="stylesheet" href="/static/theme.css" />'
            in response.body)

    def test_path_param(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = DiazoMiddleware(application, {}, testfile('path_param.xml'))
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertFalse(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

        request = Request.blank('/index.html')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_custom_environ_param(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = DiazoMiddleware(application, {}, testfile('custom_param.xml'),
                              environ_param_map={'test.param1': 'someparam'})

        request = Request.blank('/')
        response = request.get_response(app)

        self.assertFalse(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

        request = Request.blank('/')
        request.environ['test.param1'] = 'value1'
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

        request = Request.blank('/')
        request.environ['test.param1'] = 'value2'
        response = request.get_response(app)

        self.assertFalse(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_custom_param(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)
            return [HTML]

        app = DiazoMiddleware(application, {}, testfile('custom_param.xml'),
                              someparam='value1')
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Content content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

        app = DiazoMiddleware(application, {}, testfile('custom_param.xml'),
                              someparam='value2')
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertFalse(
            b'<div id="content">Content content</div>' in response.body)
        self.assertTrue(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_subrequest(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)

            request = Request(environ)
            if request.path.endswith('/other.html'):
                return [HTML_ALTERNATIVE]
            else:
                return [HTML]

        app = DiazoMiddleware(application, {}, testfile('subrequest.xml'))
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(
            b'<div id="content">Alternative content</div>' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

    def test_esi(self):
        from diazo.wsgi import DiazoMiddleware
        from webob import Request

        def application(environ, start_response):
            status = '200 OK'
            response_headers = [('Content-Type', 'text/html')]
            start_response(status, response_headers)

            request = Request(environ)
            if request.path.endswith('/other.html'):
                return [HTML_ALTERNATIVE]
            else:
                return [HTML]

        app = DiazoMiddleware(application, {}, testfile('esi.xml'),
                              filter_xpath=True)
        request = Request.blank('/')
        response = request.get_response(app)

        self.assertTrue(b'''<esi:include src="/other.html?;'''
                        b'''filter_xpath=//*[@id%20=%20'content']">'''
                        b'''</esi:include>''' in response.body)
        self.assertFalse(
            b'<div id="content">Theme content</div>' in response.body)
        self.assertTrue(b'<title>Transformed</title>' in response.body)

        request = Request.blank(
            '''/other.html?;filter_xpath=//*[@id%20=%20'content']''')
        response = request.get_response(app)
        # Strip response body in this test due too
        # https://bugzilla.gnome.org/show_bug.cgi?id=652766
        self.assertEqual(b'<div id="content">Alternative content</div>',
                         response.body.strip())


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)
